; ===============================================================
; Get Routines
; ===============================================================

; ---------------------------------------------------------------
; GetPoke:
; Given tier and #, return location of data
; Input: HL = Pointer to tier and #
; Output: HL = Pointer to data
; Preserves DE
; ---------------------------------------------------------------

GetPoke:			; ~36 bytes
.relocate SwapZone
	push DE
	ld A, (HL)		; get Tier (1-5)
	inc HL
	ld B, (HL)		; get #
	ld HL, GetPokeDb	; get base address --> DE
	ld D, 0
	dec A
	add A, A
	ld E, A
	add HL, DE
	call LdHLIndRAM
	ex DE, HL
	
	ld H, 0
	ld L, B
	push HL
	pop BC			; BC = HL
	add HL, HL		; HL * 8 - HL = HL * 7 [BYTES_PER_PKMN]
	add HL, HL
	add HL, HL
	or A
	sbc HL, BC
	add HL, DE
	pop DE
	ret
	
GetPokeDb:
.dw fPKMN1, fPKMN2, fPKMN3, fPKMN4, fPKMN5
.endrelocate

; ---------------------------------------------------------------
; GetPokeID:
; Given tier and #, return location of data, pkmn ID
; Input: HL = Pointer to tier and #
; Output: HL = Pointer to data
;         A = pkmn number
; Preserves DE
; ---------------------------------------------------------------

GetPokeID:
.relocate SwapZone
	RCall( cGetPoke )	; get pt to pkmn data
	jp GetByte		; get species ID
.endrelocate

; ---------------------------------------------------------------
; GetName:
; Given tier and #, return pointer to name
; Input: HL = Pointer to tier and #
; Output: HL = Pointer to name
; ---------------------------------------------------------------

GetName:
.relocate SwapZone
	RCall( cGetPoke )
.endrelocate
	
GetName2:
.relocate SwapZone
	call GetByte		; get species ID
.endrelocate
	
GetName3:
.relocate SwapZone
	ld H, A		; HL = Dex + (ID * 20)
	ld L, BYTES_PER_DEX
	call MultHL
	ld DE, dPokedex
	add HL, DE
	ret
.endrelocate

; ---------------------------------------------------------------
; GetType:
; Given tier and #, return pointer to type
; Input: HL = Pointer to tier and #
; Output: HL = Pointer to Type1
; ---------------------------------------------------------------

GetType:
.relocate SwapZone
	RCall( cGetName )
.endrelocate
	
GetType2:
.relocate SwapZone
	ld DE, 9
	add HL, DE
	ret
.endrelocate
	
; ---------------------------------------------------------------
; GetTypeText:
; Given Type ID, return pointer to string
; Input: A = Type ID
; Output: HL = pointer to string
; ---------------------------------------------------------------

GetTypeText:
.relocate SwapZone
	ld HL, tarTypeC
	RCall( cGetTable2ROM )
	ret
.endrelocate

; ---------------------------------------------------------------
; GetAbility
; Given tier and # and pid, return pointer to ability
; Input: HL = Pointer to tier and #
; Output: HL = Pointer to Ability
;	   A = Ability ID
; ---------------------------------------------------------------

GetAbility:
.relocate SwapZone
	inc HL
	inc HL
	ld A, (HL)		; PID --> A
	dec HL
	dec HL
	push AF
	RCall( cGetName )
	pop AF
	ld DE, 11
	add HL, DE
	bit 1, A
	jr z, GetAbility2
	inc HL
GetAbility2:
	jp GetByte
.endrelocate

; ---------------------------------------------------------------
; GetTable2
; Given a table base address and offset, find element (2 bytes)
; Input: A = Offset in RAM
;	  HL = Base
; Output: HL = Element
; ---------------------------------------------------------------

GetTable2:
.relocate SwapZone
	ex DE, HL		; DE = HL
	ld H, 0
	ld L, A		; HL = zero extended A
	add HL, HL		; HL = HL * 2
	add HL, DE		; HL = pt to table entry
	jp LdHLIndRAM		; HL = element
.endrelocate

; ---------------------------------------------------------------
; GetTable2ROM
; Given a table base address and offset, find element (2 bytes)
; Input: A = Offset in ROM
;	  HL = Base
; Output: HL = Element
; ---------------------------------------------------------------

GetTable2ROM:
.relocate SwapZone
	ex DE, HL		; DE = HL
	ld H, 0
	ld L, A		; HL = zero extended A
	add HL, HL		; HL = HL * 2
	add HL, DE		; HL = pt to table entry in ROM
	jp LdHLIndROM		; HL = element
.endrelocate

; ---------------------------------------------------------------
; GetAbilityText
; Given ability ID, return pointer to text string
; Input: A = Ability ID
; Output: HL = Pointer to string
; ---------------------------------------------------------------

GetAbilityText:
.relocate SwapZone
	ld HL, tarAbil
	RCall( cGetTable2ROM )
	ret
.endrelocate

; ---------------------------------------------------------------
; GetAbilityDesc
; Given ability ID, return pointer to description string
; Input: A = Ability ID
; Output: HL = Pointer to desc string
; ---------------------------------------------------------------

GetAbilityDesc:
.relocate SwapZone
	ld HL, tarAbilDesc
	RCall( cGetTable2ROM )
	ret
.endrelocate

; ---------------------------------------------------------------
; GetNature
; Given tier and #, return pointer to nature
; Input: HL = Pointer to tier and #
; Output: HL = Pointer to Nature
;	   A = Nature ID
; ---------------------------------------------------------------

GetNature:
.relocate SwapZone
	RCall( cGetPoke )
	ld BC, 6
	add HL, BC
	call GetByte
	and %00011111		; bitmask off 3 ev bits
	ret
.endrelocate

; ---------------------------------------------------------------
; GetNatureText
; Given nature ID, return pointer to text string
; Input: A = Nature ID
; Output: HL = Pointer to string
; ---------------------------------------------------------------

GetNatureText:
.relocate SwapZone
	ld HL, tarNature
	RCall( cGetTable2ROM )
	ret
.endrelocate

; ---------------------------------------------------------------
; GetItem
; Given tier and #, return pointer to item
; Input: HL = Pointer to tier and #
; Output: HL = Pointer to Item
;	   A = Item ID
; ---------------------------------------------------------------

GetItem:
.relocate SwapZone
	RCall( cGetPoke )
	ld BC, 5
	add HL, BC
	call GetByte
	srl A
	srl A
	srl A			; A >> 3, remove ev bits
	ret
.endrelocate

; ---------------------------------------------------------------
; GetItemText
; Given item ID, return pointer to text string
; Input: A = Item ID
; Output: HL = Pointer to string
; ---------------------------------------------------------------

GetItemText:
.relocate SwapZone
	ld HL, tarItem
	RCall( cGetTable2ROM )
	ret
.endrelocate

; ---------------------------------------------------------------
; GetMove
; Given tier and #, return pointer to move1
; Input: HL = Pointer to tier and #
; Output: HL = Pointer to Move1
; ---------------------------------------------------------------

GetMove:
.relocate SwapZone
	RCall( cGetPoke )
	inc HL
	ret
.endrelocate

; ---------------------------------------------------------------
; GetMoveText
; Given Move ID, return pointer to text string
; Input: A = Move ID
; Output: HL = Pointer to string
; ---------------------------------------------------------------

GetMoveText:
.relocate SwapZone
	ld HL, tarMove
	RCall( cGetTable2ROM )
	ret
.endrelocate

; ---------------------------------------------------------------
; GetMoveDesc
; Given Move ID, return pointer to description string
; Input: A = Move ID
; Output: HL = Pointer to string
; ---------------------------------------------------------------

GetMoveDesc:
.relocate SwapZone
	RCall( cGetMoveEffect )
.endrelocate

GetMoveDesc2:
.relocate SwapZone
	ld HL, tarMoveDesc
	RCall( cGetTable2ROM )
	ret
.endrelocate
	
; ---------------------------------------------------------------
; GetMoveType
; Given move ID, return type ID
; Input: A = Move ID
; Output: A = type ID
; ---------------------------------------------------------------

GetMoveType:				; 11 bytes
.relocate SwapZone
	call $ + 75			; RCall( cGetMoveData )
	ld A, (HL)	; jp GetByte
	srl A
	srl A
	srl A
	ret
.endrelocate

; ---------------------------------------------------------------
; GetMovePow
; Given move ID, return power value
; Input: A = Move ID
; Output: A = Power value
; ---------------------------------------------------------------

GetMovePow:				; 24 bytes
.relocate SwapZone
	call $ + 64			; RCall( cGetMoveData )
	ld D, (HL)	; jp GetByte
	inc HL
	ld A, (HL)
	srl D \ rr A
	srl D \ rr A
	srl D \ rr A
	srl A \ srl A
	jr $ + 52 + 15 + 2		; GetMoveA5, w/o bitmask
.endrelocate

; ---------------------------------------------------------------
; GetMoveAcc
; Given move ID, return accuracy value
; Input: A = Move ID
; Output: A = Accuracy value
; ---------------------------------------------------------------

GetMoveAcc:
.relocate SwapZone			; 9 bytes
	call $ + 40			; RCall( cGetMoveData )
	inc HL
	ld A, (HL)	; jp GetByte
	and %00011111
	jr $ + 45 + 15		; GetMoveA5
.endrelocate
	
; ---------------------------------------------------------------
; GetMoveEffect
; Given move ID, return effect ID
; Input: A = Move ID
; Output: A = Effect ID
; ---------------------------------------------------------------

GetMoveEffect:			; 7 bytes
.relocate SwapZone
	call $ + 31			; RCall( cGetMoveData )
	inc HL \ inc HL
	ld A, (HL) \ ret ; jp GetByte
.endrelocate
	
; ---------------------------------------------------------------
; GetMoveEffAcc
; Given move ID, return effect accuracy value
; Input: A = Move ID
; Output: A = Effect accuracy
; ---------------------------------------------------------------

GetMoveEffAcc:			; 12 bytes
.relocate SwapZone
	call $ + 24			; RCall( cGetMoveData )
	inc HL \ inc HL \ inc HL
	ld A, (HL)	; jp GetByte
	rra
	rra
	rra
	jr $ + 24 + 15		; GetMoveA5
.endrelocate
	
; ---------------------------------------------------------------
; GetMovePP
; Given move ID, return max PP value
; Input: A = Move ID
; Output: A = PP value
; ---------------------------------------------------------------

GetMovePP:				; 12 bytes
.relocate SwapZone
	call $ + 12			; RCall( cGetMoveData )
	inc HL \ inc HL \ inc HL
	ld A, (HL)	; jp GetByte
	and %00000111
	inc A
	jr $ + 12 + 15		; GetMoveA5
.endrelocate

; ---------------------------------------------------------------
; GetMoveData
; Given move ID, return pointer to data
; Input: A = Move ID
; Output: HL = Pointer to data
; ---------------------------------------------------------------

GetMoveData:				; 25 bytes total
.relocate SwapZone
	ld H, 0
	ld L, A
	add HL, HL			; A * 4
	add HL, HL
	ld DE, mdMoves		; HL = mdMoves + A * 4	
	add HL, DE
	
	ld DE, Buffer			; copy ROM move data into RAM buffer
	ld B, 4			; 4 bytes per move
	
	push DE
GetMoveDataLp:
	call GetByte			; A = byte at ROM offset, BC, DE preserved
	ld (DE), A			; store byte into buffer
	inc HL
	inc DE
	djnz GetMoveDataLp		; loop until B = 0
	pop HL				; HL = pt to start of buffer
	ret
	
GetMoveA5:				; subroutine - remove top 3 bits of A, multiplies A by 5, input: A, output: A * 5, destroys D (7 bytes)
	and %00011111
	ld D, A
	add A, A
	add A, A
	add A, D	; A * 5
	ret
.endrelocate

; ---------------------------------------------------------------
; GetMoveFlag
; Given move ID, return flag byte
; Input: A = Move ID
; Output: A = Flag byte
; Preserves BC
; ---------------------------------------------------------------

GetMoveFlag:
.relocate SwapZone
	ld D, 0
	ld E, A
	ld HL, mfFlags
	add HL, DE		; HL = offset in mfFlags table
	jp GetByte
.endrelocate
	
; ---------------------------------------------------------------
; GetMoveProtect
; Given move ID, return protect flag bit
; Input: A = Move ID
; Output: z = no protect, nz = protect
; Preserves A
; ---------------------------------------------------------------

GetMoveProtect:
.relocate SwapZone
	push AF
	RCall( cGetMoveFlag )
	and mfProtect		; bitmask AND to get protect bit
	pop BC
	ld A, B
	ret
.endrelocate

; ---------------------------------------------------------------
; GetMoveContact
; Given move ID, return contact flag bit
; Input: A = Move ID
; Output: z = no contact, nz = contact
; Preserves AF
; ---------------------------------------------------------------

GetMoveContact:
.relocate SwapZone
	push AF
	RCall( cGetMoveFlag )
	and mfContact		; bitmask AND to get contact bit
	pop BC
	ld A, B
	ret
.endrelocate

; ---------------------------------------------------------------
; GetMoveFlinch
; Given move ID, return flinch flag bit
; Input: A = Move ID
; Output: z = no flinch, nz = flinch
; Preserves AF
; ---------------------------------------------------------------

GetMoveFlinch:
.relocate SwapZone
	push AF
	RCall( cGetMoveFlag )
	and mfFlinch		; bitmask AND to get flinch bit
	pop BC
	ld A, B
	ret
.endrelocate

; ---------------------------------------------------------------
; GetMoveSound
; Given move ID, return sound flag bit
; Input: A = Move ID
; Output: z = no sound, nz = sound
; Preserves A
; ---------------------------------------------------------------

GetMoveSound:
.relocate SwapZone
	push AF
	RCall( cGetMoveFlag )
	and mfSound		; bitmask AND to get sound bit
	pop BC
	ld A, B
	ret
.endrelocate

; ---------------------------------------------------------------
; GetMoveFNT
; Given move ID, return FNT flag bit
; Input: A = Move ID
; Output: z = no FNT flag, nz = FNT flag
; Preserves AF
; ---------------------------------------------------------------

GetMoveFNT:
.relocate SwapZone
	push AF
	RCall( cGetMoveFlag )
	and mfFNT		; bitmask AND to get FNT bit
	pop BC
	ld A, B
	ret
.endrelocate