; ===============================================================
; Graphical Routines (Applied)
; ===============================================================

; ---------------------------------------------------------------
; DrawRectWhite3
; Draws a white rectangle on both buffers
; Same I/O as DrawRect
; ---------------------------------------------------------------

DrawRectWhite3:
.relocate SwapZone
	res LineColor, (IY + Asm_Flag1)	; color = white
	call $ + 3 + 4 + 1			; RCall( cDrawRectBlack )
	set LineColor, (IY + Asm_Flag1)	; color = black
	ret
.endrelocate

; ---------------------------------------------------------------
; DrawRectBlack
; Draws a black rectangle on both buffers
; Same I/O as DrawRect
; ---------------------------------------------------------------

DrawRectBlack:
.relocate SwapZone
	push HL \ push BC
	call DrawRect
	pop BC \ pop HL
	jr $ + 2 + 4 + 3 + 4 + 1
;	call $ + 3 + 1 + 4 + 3 + 4 + 1	 ; RCall( cDrawRectDark )
;	ret
.endrelocate

; ---------------------------------------------------------------
; DrawRectWhite2
; Draws a white rectangle on buffer 2
; Same I/O as DrawRect
; ---------------------------------------------------------------

DrawRectWhite2:
.relocate SwapZone
	res LineColor, (IY + Asm_Flag1)	; color = white
	call $ + 3 + 4 + 1			; RCall( cDrawRectDark )
	set LineColor, (IY + Asm_Flag1)	; color = black
	ret
.endrelocate

; ---------------------------------------------------------------
; DrawRectDark
; Draws a dark gray rectangle
; Same I/O as DrawRect
; ---------------------------------------------------------------

DrawRectDark:
.relocate SwapZone
	call GrayMode
	call DrawRect
	jp NormMode
.endrelocate

; ---------------------------------------------------------------
; DrawRectWhite
; Draws a white rectangle
; Same I/O as DrawRect
; ---------------------------------------------------------------

DrawRectWhite:
.relocate SwapZone
	res LineColor, (IY + Asm_Flag1)	; color = white
	call DrawRect
	set LineColor, (IY + Asm_Flag1)	; color = black
	ret
.endrelocate

; ---------------------------------------------------------------
; DrawBoxLbl
; Draws a box and label in gray
; Input: H = x coordinate
;	  L = y coordinate
;	  B = length (optional)
; ---------------------------------------------------------------

DrawBoxLbl:
.relocate SwapZone
	ld B, 63
.endrelocate
	
DrawBoxLbl2:
.relocate SwapZone
	push HL				; rDrawRectD(x, y, len, 7)
	push BC
	ld C, 7
	RCall( cDrawRectDark )
	pop BC
	pop HL
	ld A, L				; rVertLineD(x, y + 7, 8)
	add A, 7
	ld L, A
	push BC
	ld B, 8
	call VertLineDark
	pop BC
	push HL
	ld A, H				; rVertLineD(x + len - 1, y + 7, 8)
	add A, B
	dec A
	ld H, A
	push BC
	ld B, 8
	call VertLineDark
	pop BC
	pop HL					; rHorizLineD(x, y + 14, 63)
	ld A, L
	add A, 7
	ld L, A
	call HorizLineDark
	ret
.endrelocate
	
; ---------------------------------------------------------------
; DispType
; Displays the given type with background
; Input: A = Type ID
;	  H = x coordinate
; 	  L = y coordinate
; ---------------------------------------------------------------

DispType:
.relocate SwapZone
	push HL			; disp background
	inc H
	ld B, 25
	call DispTypeLine		; (x + 1, y)
	inc L
	dec H				; (x, y + 1)
	ld BC, 27 * 256 + 5		; len = 27, counter = 5
DispTypeLp:
	call DispTypeLine
	inc L
	dec C
	jr nz, DispTypeLp
	inc H
	ld B, 25
	call DispTypeLine
	pop HL
	
	inc H				; (PenCol, PenRow) = (H + 2, L)
	inc H
	push HL
	ld B, H
	ld H, L
	ld L, B
	ld (PenCol), HL
	
	push AF
	set TextInverse, (IY + TextFlags)	; disp name of type
	RCall( cGetTypeText )	; HL = pt to type text
	RCall( cArcTextDc2 )		; decode text, no size limit, returns HL = pt to decoded text
	push HL
	b_call(_VPutS)		; display type
	pop HL
	res TextInverse, (IY + TextFlags)
	
	pop AF				; deals w/ gray txt if phys
	cp eThreshold			; ret if spec
	pop DE
	ret nc
	ld B, 6			; rCopyBuffSect(H, L, len, 6)
	push DE
	RCall( cStringLen )
	ld B, C
	ld C, 6
	pop DE
	ex DE, HL
	push HL
	ld A, H
	RCall( cCopyBuffSect )
	pop HL				; rDrawRectWhite(H, L, 24, 6)
	ld BC, 24 * 256 + 6
	RCall( cDrawRectwhite )
	ret
	
DispTypeLine:				; subsubroutine - disps horiz line depending on A
	push AF			; A = Type ID
	cp eThreshold			; H = x coordinate
	jr c, DispTypeLinePhys	; L = y coordinate
	call HorizLine		; B = len
	pop AF				; preserves AF, BC, HL
	ret
DispTypeLinePhys:
	call HorizLineDark
	pop AF
	ret
.endrelocate
	
; ---------------------------------------------------------------
; DispGender
; Disps gender symbol as a sprite
; Input: HL = pointer to poke data
; 	  B = x coord
; 	  C = y coord
; ---------------------------------------------------------------

DispGenderCoord:			; disp gender symbol at current text coords
.relocate SwapZone
	;ld BC, (PenCol)
	ld A, (PenCol)
	ld B, A			; B = (PenCol)
	ld A, (PenRow)
	ld C, A			; C = (PenRow)
.endrelocate

DispGender:
.relocate SwapZone
	ld IX, sMaleSymb		; default = Male (1)
	push BC
	RCall( cChkGender )		; check gender bit
	pop BC
	or A
	ret z				; skip if no gender
	cp 1
	jr z, DispGenderMale		; if Male, then Skip
	ld IX, sFemaleSymb		; else, IX = Female
DispGenderMale:
	ld A, B
	ld L, C
	RCall( cgsPutSprite )
	ret
.endrelocate
	
; ---------------------------------------------------------------
; DoHeal
; Heal animation, changes HP bar
; Input: HL = amount to heal
;        (PKMNFlag) = 1 if player, 0 if opp
; ---------------------------------------------------------------

DoHeal:
.relocate SwapZone
	set HealFlag, (IY + Asm_Flag2)
	RCall( cDoDamage )
	res HealFlag, (IY + Asm_Flag2)
	ret
.endrelocate
	
; ---------------------------------------------------------------
; DoHealFlip
; Heal animation, changes HP bar, inverts PKMNFlag
; Same I/O as DoHeal
; ---------------------------------------------------------------

DoHealFlip:
.relocate SwapZone
	call FlipPKMNFlag
	RCall( cDoHeal )
	jp FlipPKMNFlag
.endrelocate
	
; ---------------------------------------------------------------
; DoDamageCalcHP
; DoDamage subroutine, calculate HP by add (heal) or sub (damage)
; Input: HL = current HP
;        BC = damage or heal amount
;        DE = pt to pkmn HP data
;        (HealFlag) = 1 if heal, 0 if damage
; Output: HL = new HP
;         z / c = set if fainted, reset otherwise
; Preserves A, DE
; ---------------------------------------------------------------

DoDamageCalcHP:
.relocate SwapZone
	bit HealFlag, (IY + Asm_Flag2)
	jr nz, DoDamageHeal
	
	;call CpHLBC			; compare HL and BC
	;push AF			; save flags
	or A				; reset c flag
	sbc HL, BC			; HL = current HP - damage, z / c flag set if fainted (HL <= BC)
	;pop AF				; get flags
	jr z, DoDamageCalcHPZ	; if damage = HP, then jump
	ret nc				; else if damage > HP, then ret w/ nc
	
DoDamageCalcHPZ:
	ld HL, 0			; if fainted (HL = BC or HL < BC), then HP = 0
	scf
	ret
	
DoDamageHeal:
	push AF			; save A (current HP bar length)
	push DE			; save pt to pkmn HP data
	push HL			; save current HP
	ld H, D
	ld L, E
	ld DE, PKMN_HP - PKMN_MaxHP
	or A
	sbc HL, DE			; HL = pt to pkmn max HP
	call LdHLIndRAM		; HL = pkmn max HP (2 byte number)
	ex DE, HL			; DE = pkmn max HP
	pop HL				; HL = current HL
	add HL, BC			; HL = current HP + heal
	call CpHLDE			; if HL < DE
	jr c, DoDamageNotOver	; then skip over
	ld H, D			; else, if HL >= DE
	ld L, E			; then HL = DE, current HP cannot exceed max HP
	
DoDamageNotOver:
	pop DE				; DE = pt to pkmn HP data
	pop AF				; z flag must be reset, b/c of jr nz
	or A				; reset c flag, can't faint w/ heal
	ret
.endrelocate
	
; ---------------------------------------------------------------
; DoDamageFlip
; damage animation, changes HP bar, inverts PKMNFlag
; Same I/O as DoDamage
; ---------------------------------------------------------------

DoDamageFlip:
.relocate SwapZone
	call FlipPKMNFlag
	RCall( cDoDamage )
	jp FlipPKMNFlag
.endrelocate
	
; ---------------------------------------------------------------
; DoDamage
; Damage animation, changes HP bar
; Input: HL = damage
; 	  (PKMNFlag) = 1 if player, 0 if opp
;        (HealFlag) = 1 if heal, 0 if damage
; Output: c = fainted
;         (Opp/PlayerFainted) = 0 | 1
; ---------------------------------------------------------------

DoDamage:				; ~188 bytes
.relocate SwapZone
DoDamage1:
	push HL
	pop BC				; BC = damage
	bit PKMNFlag, (IY + Asm_Flag1)
	jr nz, DoDamagePlayer
DoDamageOpp:
	ld HL, 18 * 256 + 12
	ld DE, OppPKMN
	jr DoDamageInit
	
DoDamagePlayer:
	ld HL, 62 * 256 + 37
	ld DE, PlayerPKMN
	
DoDamageInit:				; HL = pos to disp HP bar, BC = damage, DE = pt to data
	push HL
	push DE
	push BC
DoDamageCalcCall:
	;call $ + (DoDamageEnd - DoDamageCalcCall) + (DispHPCalc - DispHPStart)
	push DE
	ld A, H			; A = x coord, L = y coord
	call GetPixelGray		; HL = pt to pixel byte in SaveSScreen, A = bitmask
	ld B, A			; B = bitmask
	ld C, 0			; C = count, starts at 0
	
DoDamageCalcCallLp:
	ld A, (HL)			; A = pixel byte
	and B				; A = A .&. B
	or A				; if A = 0
	jr z, DoDamageCalcCallDone	; then done, return C as number of pixels in HP bar
	srl B				; else, shift B to right (B >> 1)
	jr nz, DoDamageCalcCallNZ	; if B = 0 (flags preserved)
	rr B				; then shift carry bit into leftmost bit of B
	inc HL				; then move HL to next pixel byte
	
DoDamageCalcCallNZ:
	inc C
	jr DoDamageCalcCallLp	; loop until non-black pixel found
	
DoDamageCalcCallDone:
	pop DE
	
	ld A, C			; A = current HP bar length, HL and DE preserved
	
	push AF
	ld HL, PKMN_HP
	add HL, DE			; HL = pt to pkmn HP data
	ld D, H
	ld E, L			; DE = pt to pkmn HP data
	call LdHLIndRAM		; HL = pkmn HP (2 byte number), DE not affected
	pop AF
	pop BC
	
	RCall( cDoDamageCalcHP )	; calculates HP from routine
	
	push AF			; save cur HP bar length, flags
	ld A, L
	ld (DE), A
	inc DE
	ld A, H
	ld (DE), A			; load HL into (DE), little-endian
	pop AF
	pop DE				; ld, inc DE, and push / pop do not affect z / c flag
	push AF			; preserves flags
DoDamageCalcCall2:
	call $ + (DoDamageEnd - DoDamageCalcCall2) + (DispHPCalc - DispHPStart)
	pop AF				; A = current HP bar length, B = new HP bar length
	push AF
	sub B				; A = A - B
	bit HealFlag, (IY + Asm_Flag2)
	jr z, DoDamageSkipCpl	; if heal, then take negative of A
	cpl				; one's complement
	inc A				; A = ~(A - B) + 1 = -(A - B) = B - A, two's complement
	
DoDamageSkipCpl:
	ld B, A			; B = number of pixels to clear in HP bar
	pop AF
	pop HL				; HL = pos to display HP bar
	push AF			; preserve flags
	add A, H
	ld H, A			; H = (x pos of HP bar + current HP bar length)
	
	ld A, B
	or A
	jr z, DoDamageDone		; skip if B == 0
	cp 32
	jr nc, DoDamageDone		; skip if B >= 32
	
DoDamageLp:
	push HL
	push BC
	ld A, H
	call GetPixelGray		; HL = location in buffer 2
	bit HealFlag, (IY + Asm_Flag2)
	jr nz, DoDamageHPHeal
	
	cpl				; invert bitmask
	and (HL)			; clear out one pixel
	jr DoDamageShow

DoDamageHPHeal:
	or (HL)			; add one pixel w/ OR bitmask
	
DoDamageShow:
	ld (HL), A			; store back into buffer 2
	halt \ halt \ halt
	;halt \ halt \ halt	; temp speed up
	pop BC
	pop HL
	
	bit HealFlag, (IY + Asm_Flag2)
	jr nz, DoDamageIncH		; if damage
DoDamageDecH:
	dec H				; then dec x position
	jr DoDamageChk
	
DoDamageIncH:				; else if heal
	inc H				; then inc x position
	
DoDamageChk:
	ld A, H			; Chk if HP bar out of bounds
	cp 18				; If X < 18
	jr c, DoDamageDone		; Done
	cp 91				; If X >= 91
	jr nc, DoDamageDone		; Done 
	cp 62				; If X < 62 and X > 48
	jr nc, DoDamageNotChk	; Done
	cp 48
	jr nc, DoDamageDone
DoDamageNotChk:
	
	djnz DoDamageLp
	; TODO: change shading of HP bar based on percentage left?
	
DoDamageDone:
	pop AF				; get flags
	;jr z, DoDamageFNT
	jr c, DoDamageFNT		; FNT if HL < BC or HL = BC
	ret
	
DoDamageFNT:
	bit PKMNFlag, (IY + Asm_Flag1)
	jr nz, DoDamageFntPlayer	; if player is target, player fainted
	ld HL, OppFainted		; else if player turn, opp fainted
	ld (HL), 1
	ld HL, OppPKMN + PKMN_HP	; set HP to 0
	jr DoDamageFntEnd0
	
DoDamageFntPlayer:
	ld HL, PlayerFainted
	ld (HL), 1
	ld HL, PlayerPKMN + PKMN_HP	; set HP to 0
DoDamageFntEnd0:
	ld (HL), 0
	inc HL
	ld (HL), 0
DoDamageFntEnd:
	scf				; signal FNT
	ret
DoDamageEnd:
.endrelocate

; ---------------------------------------------------------------
; DispHP
; Disps HP box w/ partial HP
; Input: A = 0 (DispHPX) | 1 (Player) | 2 (Opponent)
;        DE = pt to pkmn data
; ---------------------------------------------------------------

DispHP:
.relocate SwapZone
DispHPStart:
	jr DispHPCont			; skip over subroutine, DispHPCalc must be close to DoDamage, and inside of DispHPInit block
	
DispHPCalc:				; sub - input: DE = pt to data, output: B = HP bar len (min(29, HP / max * 30)) (0 <= B <= 29), saves HL, DE
	push HL			; save coords
	push DE
	push DE
	ld HL, PKMN_MaxHP
	add HL, DE			; HL = pt to pkmn max HP
	call LdHLIndRAM		; HL = max HP (2 byte number)
	RCall( cIntFP )		; OP1 = max HP as FP number
	rst rOP1ToOP2			; OP2 = max HP as FP number
	pop DE
	
	ld HL, PKMN_HP
	add HL, DE			; HL = pt to pkmn current HP
	call LdHLIndRAM		; HL = current HP (2 byte number)
	RCall( cIntFP )		; OP1 = current HP as FP number
	b_call(_FPDiv)		; OP1 = OP1 (current HP) / OP2 (max HP)
	rst rOP1ToOP2			; OP2 = OP1
	
	ld HL, FP30
	rst rMov9ToOp1		; OP1 = 30
	b_call(_FPMult)		; OP1 = OP1 (30) * OP2 (current HP / max HP)
	;b_call(_Int)			; round down to int
	b_call(_ConvOP1)		; DE = round(30 * (current HP / max HP)), error if OP1 exponent > 3
	ld B, E			; DE should be < 256, so D should be = 0, 0 <= B <= 30
	cp 30
	jr c, DispHpCalcLT30		; if B >= 30
	ld B, 29			; then round B down to 29
	
DispHpCalcLT30:
	pop DE
	pop HL				; get coords
	ret
DispHPCalcEnd:			; ~53 bytes to DispHPStart
	
DispHPCont:
	or A
	jr z, DispHPX
	cp 2
	jr z, DispHPOpp
DispHPPlayer:
	ld HL, 60 * 256 + 35		; (60, 35) for player
	jr DispHPInit
	
DispHPOpp:
	ld HL, 16 * 256 + 10		; (16, 10) for opponent

DispHPX:				; input: HL = pos to disp HP bar, ~110 bytes
DispHPInit:
	push DE			; save DE (pt to pkmn data)
	inc H
	push HL
	ld B, 31
	call HorizLineBlack		; (H + 1, L)
	dec H
	inc L
	ld B, 3
	call VertLineBlack		; (H, L + 1)
	push HL
	ld A, 32
	add A, H
	ld H, A
	ld B, 3
	call VertLineBlack		; (H + 32, L + 1)
	pop HL
	inc L
	inc L
	inc L
	inc H
	ld B, 31
	call HorizLineBlack		; (H + 1, L + 4)
	pop HL
	inc L
	push HL
	ld BC, 31 * 256 + 3
	call DrawRect			; (H + 1, L + 1)
	pop HL
	pop DE				; get DE (pt to pkmn data)
	
DispHPLine:
	push DE
	push HL
	ld BC, 31 * 256 + 3		; width = 31, height = 3
	
	RCall( cDrawRectWhite2 )
	;res LineColor, (IY + Asm_Flag1)	; color = white
	;call GrayMode
	;call DrawRect			; clear out second buffer (inline DrawRectWhite2)
	;call NormMode
	;set LineColor, (IY + Asm_Flag1)	; color = black
	
	pop HL
	pop DE
	
	call DispHPCalc
	inc H				; move one pixel down
	inc L				; and one pixel right
	
	call HorizLineDark
	ret
.endrelocate

; ---------------------------------------------------------------
; DispStatus
; Displays status ailment icon
; Input: A = status ailment
;        PKMNFlag
; ---------------------------------------------------------------

DispStatus:
.relocate SwapZone
	bit PKMNFlag, (IY + Asm_Flag1)
	jr nz, $ + 2 + 3 + 2 ; DispStatusPlayer
.endrelocate

DispStatusOpp:
.relocate SwapZone
	ld HL, 3 * 256 + 9		; opp coords at (3, 9)
	jr $ + 2 + 3 ; DispStatusCont
.endrelocate
	
DispStatusPlayer:
.relocate SwapZone
	ld HL, 47 * 256 + 34		; player coords at (47, 34)
.endrelocate
	
DispStatusCont:

DispStatusX:				; subroutine - display status (A) at arbitrary location (HL)
.relocate SwapZone
	or A
	jr nz, DispStatusNot0
	
DispStatus0:				; if status = 0 (stNone)
	bit InfoFlag, (IY + Asm_Flag3)
	ret nz				; if info flag set, then don't white out rectangle
	
	ld BC, 12 * 256 + 6
	RCall( cDrawRectWhite3 )	; then draw white rect on both buffers, with width = 12, height = 6
	ret
	
DispStatusNot0:
	push HL
	ld HL, StatusDb
	RCall( cGetTable2 )		; HL = pt to status text
	pop DE				; DE = coords to disp text
	
	push DE
	ld A, D
	ld (PenCol), A		; set (PenCol), (PenRow) to D, E
	ld A, E
	ld (PenRow), A
	b_call(_VPutS)
	pop HL
	
	bit InfoFlag, (IY + Asm_Flag3)
	ret nz				; if info flag set, then don't copy buffer
	
	ld A, H			; A = x coords
	ld BC, 12 * 256 + 6
	RCall( cCopyBuffSect )	; copy to buffer 2, with width = 12, height = 6
	ret
.endrelocate
	
; ---------------------------------------------------------------
; Clears textbox
; saves HL
; ---------------------------------------------------------------

ClearText:
.relocate SwapZone
	push HL
	di
	rDrawRectW3(1, 46, 94, 17) ; TODO - make faster, inline?
	;rFillBuffSect(WFill, 0, 45, 96, 19)
	rDrawBoxOut(0, 45, 96, 19)
	ei
	pop HL
	ret
.endrelocate