; ===============================================================
; Math Routines
; ===============================================================

; ---------------------------------------------------------------
; Div:
; Divides A by D
; Input: A = dividend
;	  D = divisor
; Output: L = quotient
;	   A = remainder
; ---------------------------------------------------------------

div:				; ~ 18 bytes
.relocate SwapZone
	ld H, 0
	ld L, A
	xor A
	ld B, 16
divlp:
	add HL, HL
	rla
	jr c, divv
	cp d
	jr c, divs
divv:
	sub D
	inc L
divs:
	djnz divlp
	ret
.endrelocate

; ---------------------------------------------------------------
; DivHL
; Divides HL by D (2 bytes)
; Input: HL = dividend
;	  D = divisor
; Output: L = quotient
; 	   A = remainder
; ---------------------------------------------------------------

DivHL:				; ~ 15 bytes
.relocate SwapZone
	xor A
	ld B, 16
DivHLLp:
	add HL, HL
	rla
	jr c, DivHLOverflow
	cp D
	jr c, DivHLSkip	; r < D
DivHLOverflow:
	sub D
	inc L
DivHLSkip:
	djnz DivHLLp
	ret
.endrelocate

; ---------------------------------------------------------------
; IntStr:
; Converts a 2-byte integer to a NTBS string
; Input: HL = integer to convert (< 2560)
; Output: (ConvStr) = converted string
;	   HL = pointer to conv
;	   A = str length
; ---------------------------------------------------------------

IntStr:
.relocate SwapZone
	ld C, 5		; max len = 5
	
IntStrlp:
	dec C
	ld D, 10
	RCall( cDivHL )	;HL / 10
	ld B, $30		;find ASCII equivalent of remainder
	add A, B
	push HL
	ld HL, ConvStr	;sto to (ConvStr)
	ld B, 0
	add HL, BC
	ld (HL), A
	pop HL
	ld A, L		;ret if quotient is zero
	or A
	jr nz, IntStrlp	;else, loop
	
	ld B, C
	ld A, B
	or A
	jr z, IntStr0		;skip if already left aligned
IntStrlp2:
	push BC		;shift bytes to the left until C = 0 
	ld HL, ConvStr + 1
	ld DE, ConvStr
	ld BC, 4
	ldir
	pop BC
	djnz intstrlp2
IntStr0:
	ld A, 5
	sub C
	ld HL, ConvStr
	ld B, 0
	ld C, A
	add HL, BC
	ld (HL), 0
	ld HL, ConvStr
	ret
.endrelocate
	
; ---------------------------------------------------------------
; IntFP:
; Converts a 2-byte integer to FP
; Input: HL = integer
; Output: OP1 = FP
; ---------------------------------------------------------------

IntFP:
.relocate SwapZone
	ld A, L
	or H
	jr z, IntFPZero
	RCall( cIntStr )
	call BCDFPClear		; distance to BCDFPClear must be <250 bytes
	ld HL, ConvStr
	ld DE, block18
IntFPLp:
	ld A, (HL)
	or A
	jr z, IntFPDone
	
	sub '0' - 3
	ld (DE), A
	inc DE
	inc HL
	jr IntFPLp
IntFPDone:
	xor A
	ld (DE), A
	
	jr BCDFPDone1
	;call BCDFPDone1
	;ret
	
IntFPZero:
	b_call(_OP1Set0)
	ret
	
; ---------------------------------------------------------------
; BCDFP
; Misc FP conversion routines
; Input: HL = pointer to BCD
; Output: OP1 = FP number
; ---------------------------------------------------------------

BCDFP:	
BCDFPDone1:
	ld HL, block18
	ld DE, block18A
	ld B, 0			;set counter to find first significant digit
BCDFPLp2:				;move HL to first significant, nonzero digit
	ld A, (HL)
	cp 4
	jr nc, BCDFPDone2
	or A
	jr z, BCDFPDone2
	inc HL
	inc B
	jr BCDFPLp2
BCDFPDone2:
	ld A, B
	ld (sig1), A
BCDFPLp3:
	ld A, (HL)
	inc HL
	or A
	jr z, BCDFPDone3
	sub 3				;convert each digit in block18 to TI's FP format, store in block18A
	ld (DE), A
	inc DE
	inc B
	jr BCDFPLp3
BCDFPDone3:
	ld A, B			;set length (from first sig to end of bcd)		
	ld (LengthBCD), A
	
BCDFPSkip3:				;Assume FP exponent > $80 and FP is whole #
	ld A, $80			;Exp = $80 + (length) - (sig1) - 1
	ld HL, LengthBCD
	add A, (HL)
	dec HL
	sub (HL)
	dec A
	
BCDFPExp:
	ld (block9 + 1), A		;enter exponent
	ld HL, block18A
	ld DE, block9 + 2
	ld B, 7
	
BCDFPLp4:
	push BC			;compress data in 18 byte block back into 9 byte OP stuff
	ld C, 0
	ld A, (HL)
	inc HL
	ld B, 4
BCDFPLp42:
	srl A
	rr C
	djnz BCDFPLp42
	ld A, (HL)
	inc HL
	or C
	ld (DE), A
	inc DE
	pop BC
	djnz BCDFPLp4
	
	ld HL, block9
	rst rMov9ToOp1
	ret
	
BCDFPNib:				;'subroutine' that extracts nibbles from A and outputs to CA
	ld B, 4
	ld C, 0
BCDFPNibLp:	
	sla A
	rl C
	djnz BCDFPNibLp
	ld B, 4
BCDFPNibLp2:
	srl A
	djnz BCDFPNibLp2
	ret
	
BCDFPClear:
	push HL			;clears out both 18 byte data blocks and temp data and 9 byte block
	ld HL, block9
	xor A
	ld (HL), A
	ld DE, block9 + 1
	ld BC, 18 * 2 + 3 + 9
	ldir
	pop HL
	ret
	
BCDFPZero:
	b_call(_OP1Set0)
	ret
.endrelocate