; ===============================================================
; Code
; ===============================================================

; ---------------------------------------------------------------
; Init:
; Initializes game
;  - Flags + setup
;  - Interrupt (GS)
;  - Retrieve Save
; Should only run once at beginning
; ---------------------------------------------------------------

Init:
	res FracDrawLFont, (IY + FontFlags)	; small font
	res TextInverse, (IY + TextFlags)		; no inversed text
	res TextEraseBelow, (IY + TextFlags)	; don't erase below text
	set TextWrite, (IY + sgrFlags)		; write text to buffer
	res IndicOnly, (IY + IndicFlags)		; must be reset for getkey
	res AppTextSave, (IY + AppFlags)		; to preserve TextShadow
	res CurAble, (IY + CurFlags)		; disable cursor flash
	set LineColor, (IY + Asm_Flag1)		; line color = black
	res LineInvert, (IY + Asm_Flag2)		; normal line (no invert)
	res Buffer2, (IY + Asm_Flag2)		; buffer 1 (plot)
	res SilenceFlag, (IY + Asm_Flag2)		; display text
	set FrontFlag, (IY + Asm_Flag2)		; display front sprites
	res ROMFlag, (IY + Asm_Flag1)		; data = RAM
	res BlinkerOn, (IY + Asm_Flag1)		; blinker = off
	res InfoFlag, (IY + Asm_Flag3)		; info w/ no data
	
InitGetPics:
	ld HL, PKMNPICFName				; find PKMNPICF data program
	call InitGetProg2
	ld (PKMNPICFrom), A				; save program location
	ld (PKMNPICFoff), HL
	
	ld HL, PKMNPICBName				; find PKMNPICB data program
	call InitGetProg2
	ld (PKMNPICBrom), A				; save program location
	ld (PKMNPICBoff), HL
	
InitGetSave:						; get save appvar
	call InitGetSaveFind				; find PKMNSAVE appvar
	jr c, InitGetSaveNew				; if not found, then create new save data
	
	and $1F					; A = data type
	cp AppVarObj					; if not app var
	jr nz, InitGetSaveNewDel			; then delete data, make new save data
	
	xor A
	or B						; if B = 0
	jr z, InitGetSaveRAM				; then appvar in RAM
	
	push DE					; save DE, pointer to variable data area
	ld HL, SAVESIZE				; HL = size of appvar (2 + 1 + 96)
	b_call(_EnoughMem)				; returns c if not enough memory
	pop DE						; get DE
	jr nc, InitGetSaveEnough			; skip if enough memory
	
	; // can't unarchive data
	; // TODO: if can't unarchive, then read-only, disp warning message
	
	jp Quit;;;
	
InitGetSaveEnough:
	b_call(_Arc_Unarc)				; unarchive appvar
	call InitGetSaveFind
	
InitGetSaveRAM:
	inc DE						; skip first 2 bytes
	inc DE
	ld A, (DE)					; A = ID token
	cp SAVEID					; compare to $BA
	jr nz, InitGetSaveNewDel			; if not matching, then delete
	
	inc DE						; skip ID token byte
	ex DE, HL
	ld DE, PKMNSaveData
	ld BC, SAVESIZE - 3				; BC = size of appvar - 3
	ldir
	
	ld A, (PKMNSaveRound)
	ld (RoundNum), A		; else, set round w/ saved data
	ld HL, (PKMNSaveBattle)
	ld (BattleNum), HL		; BattleNum = saved BattleNum
	ld A, (PKMNSaveMatch)
	ld (MatchNum), A		; MatchNum = saved MatchNum
	
	ld A, (PKMNSaveCurrent)
	or A
	jr z, InitGetSaveCont			; skip if forfeited match
	
	ld HL, PKMNSaveTeam
	ld DE, TeamData
	ld BC, 18
	ldir						; copy team, opp data
	
	ld HL, PKMNSavePrev
	ld DE, PrevOppTeam
	ld BC, 9
	ldir						; copy prev opp team data
	
	ld A, 1
	ld (SaveVar), A				; set savevar to 1
	
InitGetSaveCont:
	b_call(_Arc_Unarc)				; archive appvar
	jr InitInterrupt
	
InitGetSaveNewDel:
	call InitGetSaveFind				; find appvar in memory
	b_call(_DelVarArc)				; delete appvar
	
InitGetSaveNew:
	; // new save data
	xor A						; A = 0
	ld (SaveVar), A				; reset save var
	ld HL, PKMNSaveData
	ld DE, PKMNSaveData + 1
	ld BC, SAVESIZE - 3
	ld (HL), A
	ldir						; set all bytes in PKMNSaveData to 0
	
InitInterrupt:
	di
	ld HL, $9900
	ld DE, $9901
	ld BC, 256	
	ld (HL), $9A
	ldir
	ld HL, Interrupt_Start
	ld DE, $9A9A
	ld BC, Interrupt_End - Interrupt_Start ; 187 bytes
	ldir						; loads interrupt into memory
	ld A, $99
	ld I, A
	im 2
	ld A, FREQ \ out ($04), A			; set timer freq to 11
	ld A, ACKN \ out ($03), A			; acknowledge interrupt
	ld A, MASK \ out ($03), A			; set interrupt mask
	ei
	
InitText:
	ld BC, TextShadowStart
	call RCallCopy				; copy TextShadow routines into SwapZone
	
	ld HL, SwapZone
	ld DE, TextShadow
	ld BC, 128					; BC = size of TextShadow space
	ldir						; copy TextShadow routines from SwapZone into TextShadow
	
	ld BC, RProgram
	call RCallCopy				; copy main program back into SwapZone, before returning
	
	call NormMode					; normal graphics mode by default
	
	xor A						; A = 0
	ld (MenuOpt), A				; initial menu option = Play
	
	ld A, R					; A = memory refresh reg
	ld H, A					; H = memory refresh reg
	ld (symTable + 39), HL			; seed iRandom PRNG with HL
	
	ld HL, Contrast
	ld A, (HL)					; A = current contrast (0x00 - 0x24)
	ld (SaveContrast), A				; save contrast value
	ld A, 24					; A = 24, medium contrast
	ld (HL), A					; set contrast, memory location
	add A, $D8					; A = A + $D8
	call $0B					; call routine at address 0x0B
	out (16), A					; set contrast, output port
	
	ret
	
InitDataErr:
	;//handle error
	jp Quit
	
InitGetProg:						; subroutine - gets program data, input: HL = pt to name, output: A = ROM page, HL = offset
	rst rMOV9TOOP1
	b_call(_ChkFindSym)
	ret
	
InitGetProgEnd:
	add HL, DE
	call ChkROMPg
	ld A, B
	ret
	
InitGetProg1:				; subroutine - find DATA prog in archive, input: HL = pt to name, output: HL = offset, A = ROM page
	call InitGetProg
	jr c, InitDataErr		; throw error if not found
	ld HL, 20
	jr InitGetProgEnd

InitGetProg2:				; subroutine - find PIC prog in archive, input: HL = pt to name, output: HL = offset, A = ROM page
	call InitGetProg
	jr c, InitDataNoPic
	ld HL, 22
	jr InitGetProgEnd
	
InitDataNoPic:
	xor A				; A = 0
	ld H, A
	ld L, A			; HL = 0
	ret
	
InitGetSaveFind:					; subroutine - runs chkfindsym to find PKMNSAVE appvar
	ld HL, PKMNSAVEName				; HL = pt to appvar name
	rst rMov9ToOP1				; OP1 = data type + var name
	b_call(_ChkFindSym)				; searches symbol table for PKMNSAVE appvar, returns c if not found
	ret
	
InitDATA:
	ld HL, PKMNDATAName				; find PKMNDATA data program
	call InitGetProg1
	ld (PKMNDATArom), A				; save program location
	ld (PKMNDATAoff), HL
	ret
	
StackInit:				; initializes stack
	ld HL, Stack
	ld (StackPt), HL
	ld (SaveSP), SP
	ret
	
; ---------------------------------------------------------------
; MainMenu:
; Draws main menu UI and waits for user response
; Input: None
; Output: A = Menu Option
;		0 - Play
;		1 - Record
;		2 - Info
;		3 - Quit
; ---------------------------------------------------------------

MainMenu:
	di			; interrupt not needed, draws faster
	RCall( cClearBuff )
	
	rGSLargeSprite(2, 26, sBFLogo)
	rGSLargeSprite(70, 26, sKnowSymb)
	rGSLargeSprite(13, 1, sPkmnLogo)
	rGSLargeSprite(13, 13, sPkmnLogo3)
	rGSLargeSprite(28, 23, sBFTitle)
	ei
	
	di			; draw menu options text
	rVPutS(7, 57, tPlay)
	rVPutS(29, XX, tRecord)
	rVPutS(60, XX, tInfo)
	rVPutS(82, XX, tQuit)
	rCopyBuffSect(0, 58, 96, 5)
	ei
	
	ld A, (MenuOpt)
MainMenuCursor:		; A must = (MenuOpt)
	call MainMenuDispCursor
	
MainMenuLp:			; TODO: data based approach to input? (see handlekey)
	RCall( cGetInput ) ;b_call(_GetCSC)
	or A
	jr z, MainMenuLp
	cp gkEnter
	jr z, MainMenuEnd
	cp gk2nd
	jr z, MainMenuEnd
	cp gkClear
	jp z, Quit
	cp gkMode
	jp z, Quit
	cp gkRight
	jr z, MainMenuRight
	cp gkLeft
	jr z, MainMenuLeft
	
	jr MainMenuLp
	
MainMenuRight:
	ld HL, MenuOpt
	ld A, (HL)
	cp 3			; if A = 3, Quit option
	jr z, MainMenuLp	; then go back to loop, can't move right
	call MainMenuDispCursor
	inc (HL)		; else, inc (HL)
	ld A, (HL)		; A = (MenuOpt)
	jr MainMenuCursor
	
MainMenuLeft:
	ld HL, MenuOpt
	ld A, (HL)
	or A			; if A = 0, Play option
	jr z, MainMenuLp	; then go back to loop, can't move left
	call MainMenuDispCursor
	dec (HL)		; else, dec (HL)
	ld A, (HL)		; A = (MenuOpt)
	jr MainMenuCursor
	
MainMenuDispCursor:			; disp XOR pokeball cursor at (CursorTab[A], 57), saves HL
	push HL
	ld HL, MainMenuCursorTab	; get x pos from table
	ld B, 0
	ld C, A
	add HL, BC
	ld A, (HL)
	ld L, 57			
	ld IX, sPokeball
	RCall( cgsPutSprite )
	pop HL
	ret
	
MainMenuEnd:
	ld A, (MenuOpt)
	ret				; return with A = (MenuOpt)
	
MainMenuCursorTab:
.db 0, 22, 53, 75

; ---------------------------------------------------------------
; Results:
; Display results screen, and waits for keypress
; Inputs: None
; ---------------------------------------------------------------

Results:
	xor A
	ld (RecordFlag), A
	call ResultsDraw
	
ResultsLp:
	RCall( cWait )
	ret
	
ResultsDraw:				; subroutine - draws results screen, input: RecordFlag = 1 for new record, 0 = view results
	di
	RCall( cClearBuff )
	ld A, (RecordFlag)
	or A
	jr nz, ResultsNew
ResultsView:
	rArcText(5, 0, tBattleSwapSingleResults)
	jr ResultsCont
	
ResultsNew:
	rArcText(27, 0, tNewRecord)
		
ResultsCont:
	ld HL, 8			; x1 = 0, y1 = 8
	ld DE, 95 * 256 + 8		; x2 = 95, y2 = 8
	call fastlined
	ei
	
	di
	rArcText(8, 10, tName)
	rArcText(61, 10, tWins)
	rArcText(77, 10, tSwap)
	
	ld A, 16
	ld B, 1
ResultsNums:
	ld (PenRow), A		; (PenRow) = A, (16, 22, 28, 34, 40)
	add A, 6			; A += 6
	push AF
	ld A, 1
	ld (PenCol), A		; (PenCol) = 1
	ld A, '0'
	add A, B			; A = '0' + B
	b_call(_VPutMap)		; disp A, preserves BC, HL
	ld A, ')'
	b_call(_VPutMap)		; disp ')', preserves BC, HL
	inc B				; B ++
	pop AF
	cp 46
	jr c, ResultsNums
	
	ld A, 16
	ld HL, PKMNSaveData		; HL = pt to save data, name of record #1
ResultsNames:
	ld (PenRow), A		; (PenRow) = A, (16, 22, 28, 34, 40)
	add A, 6			; A += 6
	push AF
	ld A, 8
	ld (PenCol), A		; (PenCol) = 8
	ld A, (HL)			; A = first char in name
	or A				; if A is null
	jr z, ResultsNameNone	; thens skip
	push HL			; save HL
	ld B, 8			; display 8 chars max
	RCall( cPutSN )		; display name
	pop HL				; get HL

ResultsNameNone:
	ld BC, 8
	add HL, BC			; HL += 8, pt to score, skip past name
	push HL			; save HL
	call LdHLIndRAM		; HL = score
	RCall( cIntStr )		; HL = pt to score as string
	call ResultsRightWinTxt	; right align (PenCol), HL preserved
	b_call(_VPutS)		; display score
	pop HL				; get HL
	inc HL
	inc HL				; HL += 2, pt to swap, skip past score
	push HL			; save HL
	call LdHLIndRAM		; HL = swap number
	RCall( cIntStr )		; HL = pt to swap num as string
	RCall( cRightText )		; right align text
	b_call(_VPutS)		; display swap number
	pop HL				; get HL
	inc HL
	inc HL				; HL += 2, pt to next name, skip past swap num
	pop AF
	cp 46
	jr c, ResultsNames
	
	ld A, (RecordFlag)
	or A
	jr nz, ResultsSkipBack	; skip current / previous data, 'back' text if new record
	
	ld HL, tCurrent
	ld A, (PKMNSaveCurrent)
	or A
	jr nz, ResultsCurrent
	ld HL, tPrevious
	
ResultsCurrent:
	rTextCoord(8, 48)		; display text at (8, 48)
	RCall( cArcText )		; display 'Current' | 'Previous'
	ld HL, (PKMNSaveBattle)
	ld A, H
	or L				; if HL = 0
	jr z, ResultsZero		; then don't dec HL
	dec HL				; win streak = match number - 1
ResultsZero:
	RCall( cIntStr )		; HL must be < 2560, HL = pt to str, A = str length
	call ResultsRightWinTxt	; right align text
	;ld A, 48
	;ld (PenRow), A		; (PenRow) = 48
	b_call(_VPutS)		; display win streak text
	
	ld HL, (PKMNSaveSwap)
	RCall( cIntStr )		; HL must be < 2560, HL = pt to str, A = str length
	RCall( cRightText )		; right align (PenCol), HL preserved
	b_call(_VPutS)		; display swap number text
	
	rArcText(7, 57, tBack)	; display 'Back' text
	
ResultsSkipBack:
	rArcText(65, 57, tBP)	; display 'BP:' text
	ld A, (PKMNSaveBP)
	ld H, 0
	ld L, A			; HL = zero extended A, number of battle points
	RCall( cIntStr )		; HL = pt to str, A = str length
	RCall( cRightText )		; right align (PenCol), HL preserved
	b_call(_VPutS)		; display battle point number text
	
	ld HL, PlotSScreen		; copy entire buffer to make text black
	ld DE, SaveSScreen
	ld BC, 64 * 12
	ldir
	
	ld A, (RecordFlag)
	or A
	jr nz, ResultsSkipBall	; if not new record
	rGSPutSprite(0, 57, sPokeball); display pokeball sprite
	
ResultsSkipBall:
	ei
	ret
	
ResultsRightWinTxt:			; subroutine - right align win text
	RCall( cRightText )		; A = new (PenCol), HL preserved
	sub 21				; A = A - 21
	ld (PenCol), A		; set PenCol
	ret
	
; ---------------------------------------------------------------
; AboutDisp
; Display about info
; ---------------------------------------------------------------
	
AboutDisp:
	di
	rFillBuffSect(WFill, 0, 57, 96, 6)	; clear bottom options
	rVPutS(41, 38, tv10)			; display 'v 1.0'
	rVPutS(34, 44, tVictorGe)		; display name
	rCopyBuffSect(32, 39, 32, 11)	; copy to buffer 2
	rVPutS(9, 50, tvictorge10)		; display email
	rArcText(7, 57, tBack)		; display back option
	rCopyBuffSect(0, 51, 88, 12)	; copy to buffer 2
	rGSPutSprite(0, 57, sPokeball)	; display pokeball
	ei
	
	jp ResultsLp			; loop until key pressed
	
; ---------------------------------------------------------------
; HighScore
; Checks for high score, then prompts for name, saves data
; ---------------------------------------------------------------

HighScore:
	xor A
	ld (PKMNSaveCurrent), A	; invalidate current run
	ld HL, PKMNSaveWins		; HL = pt to #1 score
	ld B, 5			; 5 total scores
	ld DE, (BattleNum)		; DE = battle number
	dec DE				; DE = win streak = battle - 1
	
HighScoreLp:
	push HL			; save HL, pt to score
	call LdHLIndRAM		; HL = current high score, BC, DE preserved
	call CpHLDE			; compare HL and DE, regs preserved
	pop HL				; get HL, pt to score, c flag unaffected
	jr c, HighScoreBeat		; if HL < DE, then new high score
	ld A, B			; save B in A
	ld BC, 12			; BC = 12
	add HL, BC			; HL += 12, move to next win streak
	ld B, A			; get B
	djnz HighScoreLp		; loop until B = 0
	ret				; ret if no high scores beaten
	
HighScoreBeat:			; high score beaten, HL = pt to score, DE = score, B = counter (1 - 5)
	push BC			; save B
	push DE			; save DE
	push HL			; save HL
	ld DE, 8
	or A
	sbc HL, DE			; HL -= 12, move from pt to score to name
	ld (BufferPt), HL		; set Buffer ptr to HL
	dec B				; B = 0 - 4, number of times to shift
	jr z, HighScoreShiftSkip	; skip if B = 0
	
	ld HL, PKMNSaveData + 12 * 3
	ld DE, PKMNSaveData + 12 * 4
HighScoreShiftLp:
	push BC			; save B
	ld BC, 12			; 12 bytes per save data
	ldir				; copy data down
	ld BC, 24
	or A				; reset carry flag
	sbc HL, BC			; HL = HL - 24, move up to next save data
	ex DE, HL
	or A
	sbc HL, BC
	ex DE, HL			; DE = DE - 24, move up to next save data
	pop BC				; get B
	djnz HighScoreShiftLp	; loop until B = 0
	
HighScoreShiftSkip:
	pop HL
	pop DE
	call HighScoreStore		; store new score
	ld DE, (PKMNSaveSwap)	; DE = swap number
	call HighScoreStore		; store new swap number
	
	ld BC, 12			; BC = 12
	xor A				; A = 0, reset c flag
	sbc HL, BC			; HL -= 12, HL pts to name bytes
	ld (HL), A			; set first byte to 0
	push HL
	pop DE
	inc DE				; DE = HL + 1
	ld BC, 7			; clear next 7 bytes
	ldir				; clear previous name
	
	ld A, 1
	ld (RecordFlag), A
	call ResultsDraw		; draw results screen
	
	pop BC				; get B = counter (1 - 5)
	
	; initialize input
	ld A, 8
	ld (PenCol), A		; (PenCol) = 8
	ld A, 5
	sub B
	ld B, A			; A = B = 5 - counter, (0 - 4)
	add A, A
	add A, B
	add A, A			; A = B * 6
	add A, 16			; A = 16 + B * 6
	ld (PenRow), A		; (PenRow) = 16 + B * 6
	xor A
	ld (BuffSize), A		; set (BuffSize) to 0, empty
	ld (AlphaNum), A		; set (AlphaNum) to 0, uppercase
	
HighScoreInCur:
	call BuffCur			; set default cursor to '|'
	
HighScoreInCursor:
	call CursorOff
	call CursorOn
	
HighScoreInLp:
	call GetInputCur
	cp gkEnter
	jr z, HighScoreInDone
	cp gkDel
	jr z, HighScoreInDel
	cp gkClear
	jr z, HighScoreInClear
	cp gkLeft
	jr z, HighScoreInDel
	cp gk2nd
	jr z, HighScore2nd
	cp gkAlpha
	jr z, HighScoreAlpha
	cp gkMode
	jp z, Quit
	call GetChar			; convert key code to char code
	jr c, HighScoreInLp		; if error, then go back to loop
	call PutBuffer		; else, store char in buffer
	jr z, HighScoreInLp		; if full buffer, then go back to loop
	call GPutMap			; display char
	jr HighScoreInLp
	
HighScoreInDel:
	call BuffCur
	call BuffDel
	jr HighScoreInLp
	
HighScoreInClear:
	call BuffCur
	call BuffClear
	jr HighScoreInLp
	
HighScoreInDone:
	ld A, (BuffSize)
	ld B, A			; B = A = (BuffSize)
	call GetBufferLocA		; HL = pt to current location in buffer, destroys A
	xor A				; A = 0
	cp B				; if nothing in buffer
	jr z, HighScoreInLp		; then go back to loop
	
	ld A, 8
	sub B
	ret z				; ret if 8 bytes in name
	ld B, A			; B = A = 8 - (BuffSize)
	xor A				; A = 0, null char
	
HighScoreInDoneLp:
	ld (HL), A			; add null char to buffer
	inc HL				; move to next char
	djnz HighScoreInDoneLp	; loop until B = 0
	ret				; ret to main code
	
HighScore2nd:
	ld A, (BuffSize)
	cp 8
	jr z, HighScoreInLp
	
	ld HL, AlphaNum
	ld A, 2			; A = 2, numerical
	cp (HL)			; if num status
	jr z, HighScore2ndNum	; then set to uppercase
	ld (HL), A			; else set to numerical
	jr HighScoreInCur
	
HighScoreAlphaLower:
HighScore2ndNum:
	xor A
	ld (HL), A			; set (AlphaNum) to 0, uppercase
	jr HighScoreInCur
	
HighScoreAlpha:
	ld A, (BuffSize)
	cp 8
	jr z, HighScoreInLp
	
	ld HL, AlphaNum
	ld A, 1			; A = 1, lowercase
	cp (HL)			; if lowercase status
	jr z, HighScoreAlphaLower	; then set to uppercase
	ld (HL), A			; else set to lowercase
	jp HighScoreInCur
	
HighScoreStore:			; subroutine - stores DE in (HL), little endian, HL += 2
	ld A, E
	ld (HL), A
	inc HL
	ld A, D
	ld (HL), A			; (HL) = DE, little endian
	inc HL				; HL += 2, HL = pt to swap number
	ret
	
BuffCur:				; subroutine - set (CursorChar), depending on (AlphaNum)
	ld DE, CursorChar
	ld A, (AlphaNum)
	or A
	jr z, BuffCurUpper
	dec A
	jr z, BuffCurLower
	;jr z, BuffCurNum
	
BuffCurNum:				; if number
	ld A, $1F			; then set (CursorChar) to arrow
	ld (DE), A
	ret
	
BuffCurUpper:				; if uppercase
	ld A, $7C			; then set (CursorChar) to bar '|'
	ld (DE), A
	ret
	
BuffCurLower:				; if lowercase
	ld A, $5F			; then set (CursorChar) to underscore '_'
	ld (DE), A
	ret
	
PutBuffer:				; subroutine - saves char to buffer, input: A = char, (BuffSize), preserves A
	ld B, A			; save A in B
	ld HL, BuffSize		; HL = pt to BuffSize
	ld A, 8
	cp (HL)
	ret z				; ret if buffer at max size, 8 chars
	dec A
	cp (HL)			; if buffer to be filled, 7 chars
	jr nz, PutBufferNotFull
	ld A, $F1
	ld (CursorChar), A		; then set (CursorChar) to dither fill
	
PutBufferNotFull:
	ld D, 0
	ld E, (HL)			; DE = (BuffSize)
	inc (HL)			; increment size
	call GetBufferLoc		; HL = current pos in buffer
	ld (HL), B			; store char in Buffer
	ld A, B			; restore A
	ret				; ret w/ nz
	
GetBuffer:				; subroutine - gets current char in buffer
	ld A, (BuffSize)
	call GetBufferLocA		; HL pts to current char in buffer
	ld A, (HL)			; A = current char
	ret
	
GetBufferLocA:			; subroutine - get current location in buffer, input: A = offset
	ld D, 0
	ld E, A
	
GetBufferLoc:				; subroutine - get current location in buffer, input: DE = offset
	ld HL, (BufferPt)
	add HL, DE
	ret
	
BuffDel:				; subroutine - deletes last char, clears char
	ld HL, BuffSize
	xor A
	cp (HL)
	ret z				; go back if nothing in buffer
	dec (HL)			; decrement size
	call CursorOff		; clear cursor
	call GetBuffer		; get current char in buffer
	ld (HL), 0			; clear current char in buffer
	b_call(_LoadPattern)		; get size of current char
	ld HL, sFont_record
	ld B, (HL)			; B = size in pixels
	ld HL, PenCol
	ld A, (HL)
	sub B				; A = (PenCol) - char size
	ld (HL), A			; set (PenCol) to A
	ld B, A			; save A in B
	ld A, $06
	call GPutMap			; display 4 pixel ' ', clears char
	ld A, $20
	call GPutMap			; display 1 pixel ' ', clears 5 pixel chars
	ld (HL), B			; restore (PenCol)
	ret
	
BuffClear:				; subroutine - clears buffer, clears display area
	ld HL, BuffSize		; HL = pt to buffer size
	ld B, (HL)			; B = current size
	xor A
	cp (HL)
	ret z				; ret if size is 0
	ld (HL), A			; else set size to 0
	push BC			; save previous size
	call CursorOff		; clear cursor
	pop BC				; B = previous size
	ld HL, PenCol
	ld C, 8
	ld (HL), C			; (PenCol) = 8
	
BuffClearLp:				; display 6 pixel space for every char
	ld A, $06
	b_call(_VPutMap)		; saves HL
	ld A, $20
	b_call(_VPutMap)
	ld A, $20
	b_call(_VPutMap)			
	djnz BuffClearLp		; loop until B = 0
	call GPutMapCopy		; copy buffer, saves HL, BC
	
	ld (HL), C			; (PenCol) = 8
	xor A				; A = 0
	ld HL, (BufferPt)		; HL = pt to start of buffer
	ld (HL), A			; set first byte to 0
	push HL
	pop DE
	inc DE				; DE = HL + 1
	ld BC, 7
	ldir				; clear entire buffer, 8 bytes
	
	jr CursorOn			; display cursor
	
GetChar:				; subroutine - converts key code to char code, input: A = key code, output: A = char code, c if error
	sub gk0
	ret c				; ret if A is less than gk0
	
	ld D, 0
	ld E, A			; DE = zero extended A
	ld BC, AlphaNum
	ld A, (BC)			; A = alpha num status
	cp 1
	jr z, GetCharLow
	cp 2				; if num status
	jr z, GetCharNum		; then HL = num table
	ld HL, CharTable		; else HL = char table
	jr GetCharCont
	
GetCharLow:
	ld HL, LowTable
	jr GetCharCont
	
GetCharNum:
	ld HL, NumTable
	
GetCharCont:
	add HL, DE
	ld A, (HL)			; A = char code
	or A				; if A != 0
	ret nz				; then ret w/ nc, no error
	scf				; else, set c flag
	ret				; return w/ error
	
GPutMap:				; subroutine - grayscale vputmap, copies buffer after put char, input: A = char, preserves BC, HL
	b_call(_VPutMap)		; saves HL, BC
	
GPutMapCopy:				; copy from buffer 1 to buffer 2
	push BC
	push HL
	ld HL, PlotSScreen + 17 * 12
	ld DE, SaveSScreen + 17 * 12
	ld BC, 29 * 12
	ldir				; copy from Plot to Save screen, 96 x 26 rectangle at (0, 17)
	pop HL
	pop BC
	ret
	
CursorOn:				; subroutine - puts cursor, backs up PenCol
	set CursorFlag, (IY + Asm_Flag3)
	ld A, 10
	ld (CursorCount), A
	ld HL, PenCol
	ld B, (HL)			; save current column in B
	ld A, (CursorChar)
	call GPutMap			; put '|' char, saves BC, HL
	ld (HL), B			; restore column
	ret
	
CursorOff:				; subroutine - erases cursor, backs up PenCol
	res CursorFlag, (IY + Asm_Flag3)
	ld A, 10
	ld (CursorCount), A
	ld HL, PenCol
	ld B, (HL)			; saves current column in B
	ld A, $06
	call GPutMap			; put ' ' space, saves BC, HL
	ld (HL), B			; restore column
	ret
	
GetInputCur:				; subroutine - gets input, blinks char, output: A = key code
	halt \ halt \ halt
	ld HL, CursorCount
	dec (HL)			; if CursorCount -- != 0
	jr nz, GetInputCurLp		; then skip
	bit CursorFlag, (IY + Asm_Flag3); else, if CursorFlag is set
	jr nz, GetInputCurOff	; then clear cursor
	call CursorOn			; else, display cursor
	jr GetInputCurLp
	
GetInputCurOff:
	call CursorOff		; clear cursor
	
GetInputCurLp:
	RCall( cGetInput )
	or A
	jr z, GetInputCur
	ret
	
; ---------------------------------------------------------------
; SelectRound:
; Draws UI and asks user to select round (1-5)
; Inputs: None
; Outputs: A = Match #
;	      = $FF if returning to main menu
; ---------------------------------------------------------------

SelectRound:
	ld A, (PKMNSaveCurrent)
	or A				; if currently in battle
	ret nz				; then ret
	
	;ld A, (SaveVar)
	;or A				; if round number of save data is not 0
	;ret nz				; then return, else new round
	
SelectRoundNew:
	di
	RCall( cClearBuff )
	
	ld A, 13
SelectRoundUILp:			; For(A, 13, 53, 10)
	push AF			; DrawBox(0, A, 96, 10)
	ld H, 0			; End
	ld L, A
	ld BC, 96 * 256 + 10
	RCall( cDrawBox )
	pop AF
	add A, 10
	cp 53 + 10
	jr nz, SelectRoundUILp
	
	ld A, 0
SelectRoundTxtLp:			; For(A, 0, 4)
	push AF			; Y = A * 10 + 15
	ld H, A			; Text(4, Y, "Round")
	ld L, 10			; Text(A + 1)
	call MultHL			; End
	ld A, L
	add A, 15
	ld (PenRow), A
	ld A, 4
	ld (PenCol), A
	ld HL, txtRound
	RCall( cArcText )
	ld A, ' '
	b_call(_VPutMap)
	pop AF
	push AF
	add A, '1'
	b_call(_VPutMap)
	pop AF
	inc A
	cp 5
	jr nz, SelectRoundTxtLp
	
	rArcText(27, 3, tSelectRound)
	rCopyBuffSect(24, 4, 48, 5)
	ei
	
	ld A, 1
	ld (SelectRnum), A
	
SelectRoundCursor:
	ld B, $FF
	call SelectRoundFill
	
SelectRoundLp:
	RCall( cGetInput ) ;b_call(_GetCSC)
	or A
	jr z, SelectRoundLp
	cp gkEnter
	jr z, SelectRoundEnd
	cp gk2nd
	jr z, SelectRoundEnd
	cp gkClear
	jr z, SelectRoundEndMM
	cp gkAlpha
	jr z, SelectRoundEndMM
	cp gkMode
	jp z, Quit
	cp gkUp
	jr z, SelectRoundUp
	cp gkDown
	jr z, SelectRoundDown
	
	jr SelectRoundLp
	
SelectRoundUp:
	ld A, (SelectRnum)
	cp 1
	jr z, SelectRoundLp
	ld B, 0
	call SelectRoundFill
	call SelectRoundRestore
	ld A, (SelectRnum)
	dec A
	ld (SelectRnum), A
	jr SelectRoundCursor
	
SelectRoundDown:
	ld A, (SelectRnum)
	cp 5
	jr z, SelectRoundLp
	ld B, 0
	call SelectRoundFill
	call SelectRoundRestore
	ld A, (SelectRnum)
	inc A
	ld (SelectRnum), A
	jr SelectRoundCursor
	
SelectRoundEnd:
	ld A, (SelectRnum)
	ld (RoundNum), A	; A --> Round; code originally in main loop
	ld HL, 1
	ld (BattleNum), HL	; 1 --> Battle
	ld A, 1
	ld (MatchNum), A	; 1 --> Match
	xor A
	ld (PKMNSaveSwap), A	; 0 --> Swap number
	ld (SaveVar), A	; 0 --> Save var
	RCall( cCopySave )
	ret

SelectRoundEndMM:
	ld A, $FF
	ret
	
SelectRoundFill:			; Routine - Fills in textbox
	ld H, A			; Input: A = Round # (1-5)
	ld L, 10			;	  B = Fill byte (00 or FF)
	push BC
	call MultHL
	ld A, 3
	add A, L
	ld H, A			; Y = A * 10 + 3
	ld L, 12
	call MultHL
	ld BC, SaveSScreen
	add HL, BC			; HL = Y * 12 + Buffer2
	pop BC
	ld (HL), B
	push HL
	pop DE
	inc DE				; DE = HL + 1
	ld BC, 12 * 10 - 1
	ldir
	ret
	
SelectRoundRestore:			; Routine - Restores a box
	di
	ld A, (SelectRnum)		; Input: (SelectRnum) = Round # (1-5)
	ld H, A
	ld L, 10
	call MultHL
	ld A, 3
	add A, L
	ld H, 0
	ld L, A
	ld BC, 96 * 256 + 10
	RCall( cDrawBox )
	ei
	ret
	